/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import javafx.ui.RotatableWidget;
import javafx.ui.ActionWidget;
import javafx.ui.KeyStroke;
import javafx.ui.Icon;
import javafx.ui.HorizontalAlignment;
import javafx.ui.VerticalAlignment;
import javafx.ui.Insets;

/**
 * An implementation of a "push" button. Encapsulates javax.swing.JButton.
 */

public class Button extends Widget, RotatableWidget, ActionWidget {
    private attribute button: <<net.java.javafx.ui.XButton>>;
    /** Determines whether this is the default button within a dialog. */
    public attribute defaultButton: Boolean?;
    /** Determines whether this is the default cancel button within a dialog. */
    public attribute defaultCancelButton: Boolean?;
    /** Sets this button's text. */
    public attribute text: String?;
    /** Sets this button's mnemonic. */
    public attribute mnemonic: KeyStroke?;
    /** Sets this button's default icon. */
    public attribute icon: Icon?;
    /** Sets this button's selected icon. */
    public attribute selectedIcon: Icon?;
    /** Sets this button's pressed icon. */
    public attribute pressedIcon: Icon?;
    /** Sets this button's rollover icon. */
    public attribute rolloverIcon: Icon?;
    /** Sets this button's selected rollover icon. */
    public attribute rolloverSelectedIcon: Icon?;
    /** Sets whether rollover effects are enabled for this button. Defaults to true. */
    public attribute rolloverEnabled: Boolean;
    /** Sets this button's disabled icon. */
    public attribute disabledIcon: Icon?;
    /** Sets this button's disabled selected icon. */
    public attribute disabledSelectedIcon: Icon?;
    /**
     * Sets the amount of space between the text and the icon
     * displayed in this button.
     */
    public attribute iconTextGap: Number?;
    /**
     * Sets whether this button will paint the content
     * area.  If you wish to have a transparent button, such as
     * an icon-only button, for example, then you should set
     * this to <code>false</code>. Defaults to <code>true</code>.
     */
    public attribute contentAreaFilled: Boolean?;
    /**
     * Sets whether the focus state is painted for this button.
     * The default value for the <code>focusPainted</code> attribute
     * is <code>true</code>.
     * Some look and feels might not paint focus state;
     * they will ignore this property.
     */
    public attribute focusPainted:Boolean?;
    /**
     * Sets whether (if this button has a border) the border is painted.
     * Defaults to true.
     */
    public attribute borderPainted:Boolean?;
    /**
     * Sets space for margin between this button's border and
     * the label. Setting to <code>null</code> will cause this button to
     * use the default margin.  The button's default <code>Border</code>
     * object will use this value to create the proper margin.
     * However, if a non-default border is set on the button, 
     * it is that <code>Border</code> object's responsibility to create the
     * appropriate margin space (else this property will
     * effectively be ignored).
     */
    public attribute margin:Insets?;
    /**
     * Sets the horizontal position of the text relative to the icon.<br></br>
     * One of the following values:
     * <ul>
     * <li>CENTER
     * <li>LEADING
     * <li>TRAILING (the default)
     * </ul>
     */
    public attribute horizontalTextPosition: HorizontalAlignment?;
    /**
     * Sets the vertical position of the text relative to the icon.<br></br>
     * One of the following values:
     * <ul>
     * <li>CENTER (the default)
     * <li>TOP
     * <li>BOTTOM
     * </ul>
     */
    public attribute verticalTextPosition: VerticalAlignment?;
    /**
     * Sets the horizontal alignment of the the icon and text.<br></br>
     * One of the following values:
     * <ul>
     * <li>CENTER
     * <li>LEADING
     * <li>TRAILING (the default)
     * </ul>
     */
    public attribute horizontalAlignment: HorizontalAlignment?;
    /**
     * Sets the vertical alignment of the icon and text.<br></br>

     * One of the following values:
     * <ul>
     * <li>CENTER (the default)
     * <li>TOP
     * <li>BOTTOM
     * </ul>
     */
    public attribute verticalAlignment: VerticalAlignment?;
}

attribute Button.opaque = true;

operation Button.setRotation(n:Number) {
    button.setRotation(n);
}

trigger on Button.rolloverEnabled = value {
    button.setRolloverEnabled(value);
}

trigger on (Button.horizontalTextPosition = a) {
    button.setHorizontalTextPosition(a.id);
    button.revalidate();
}

trigger on (Button.verticalTextPosition = a) {
    button.setVerticalTextPosition(a.id);
    button.revalidate();
}

trigger on (Button.horizontalAlignment = a) {
    button.setHorizontalAlignment(a.id);
    button.revalidate();
}

trigger on (Button.verticalAlignment = a) {
    button.setVerticalAlignment(a.id);
    button.revalidate();
}

trigger on (Button.focusPainted = value) {
    if (value <> null) {
        button.setFocusPainted(value);
        button.revalidate();
    }
}

trigger on (Button.borderPainted = value) {
    if (value <> null) {
        button.setBorderPainted(value);
        button.revalidate();
    }
}

trigger on (Button.contentAreaFilled = value) {
    if (value <> null) {
        button.setContentAreaFilled(value);
        button.revalidate();
    }
}

trigger on (Button.icon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    button.setIcon(icon);
    if (selectedIcon == null) {
        button.setSelectedIcon(icon);
    }
}

trigger on (Button.margin = value) {
    button.setMargin(value.awtinsets);
    button.revalidate();
}

trigger on (Button.rolloverIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    button.setRolloverIcon(icon);
}

trigger on (Button.rolloverSelectedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    button.setRolloverSelectedIcon(icon);
}

trigger on (Button.selectedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    button.setSelectedIcon(icon);
}

trigger on (Button.pressedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    button.setPressedIcon(icon);
}

trigger on (Button.disabledIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    button.setDisabledIcon(icon);
}

trigger on (Button.disabledSelectedIcon = value) {
    var icon = null;
    if (value <> null) {
        icon = value.getIcon();
    }
    button.setDisabledSelectedIcon(icon);
}

trigger on (Button.iconTextGap = value) {
    button.setIconTextGap(value);
}

trigger on (Button.defaultButton = value) {
    button.setDefault(value);
}

trigger on (Button.defaultCancelButton = value) {
    button.setDefaultClose(value);
}

operation Button.createComponent():<<javax.swing.JComponent>> {
    return button;
}

trigger on (new Button) {
    opaque = true;
    button = context:<<net.java.javafx.ui.UIContext>>.createButton();
    enabled = true;
    button.setOpaque(true);
    var zb = this;
    button.addActionListener(new <<java.awt.event.ActionListener>> {
                                 operation actionPerformed(e:<<java.awt.event.ActionEvent>>) {

                                         (zb.action)();

                                 }
                             });
}


trigger on (Button.mnemonic = value) {
    button.setMnemonic(value.id);
}

trigger on (Button.text = value) {
    button.setText(value);
}
